"""Python API for FlashXTest"""

#-------------------------------------------------------------------------------
#
# FlashTest does not communicate success or failure using the error code.
#
# This script
# 1) sets up a clean test environment,
# 2) runs all desired tests in a single FlashTest run regardless of the success
#    of previous tests,
#
#-------------------------------------------------------------------------------

import os

from .. import resources

def FlashXTest(testDict,**attributes):
    """
    Python API to FlashXTest

    Arguments
    ---------
    testDict : {testName : testList}
             
                 testName : Name of the test xml file
                 testList : Tests to run from xml file

    attributes  : Dictionary to override values from Config file
    """

    # Setup enivornment
    testDir = os.getenv('PWD')

    # Main dictionary from Config file
    mainDict = __parseFile('{0}/Config'.format(testDir),**attributes)

    # Build sfocu
    __buildSFOCU(mainDict)

    # Run FlashTest
    __flashTest(testDir,testDict,mainDict)

def __parseFile(configFile,**attributes):
    """
    parse Config file to get flashDir and site info

    Arguments
    --------
    configFile  : Path to config file for test
    attributes  : Dictionary to override values from Config file

    Returns
    -------
    mainDict: Dictionary for keys in the config file
    """
    mainDict=resources.FlashTest.lib.parseFile(configFile)

    for key,value in attributes.items(): 
        if value and key in mainDict:
            mainDict[key] = value

    return mainDict

def __buildSFOCU(mainDict):
    """
    Build SFOCU (Serial Flash Output Comparison Utility)

    Arguments:
    ----------
    mainDict: Dictionary from Config file
    """
    # Build brand new version of sfocu
    os.chdir('{0}/tools/sfocu'.format(mainDict['pathToFlash']))
    os.system('make SITE={0} NO_NCDF=True sfocu clean'.format(mainDict['flashSite']))
    os.system('make SITE={0} NO_NCDF=True sfocu'.format(mainDict['flashSite']))

def __getTestInfo(testDir,outDir,testName,flashSite):
    """
    Get test info site

    Arguments:
    -----------
    testDir   : Test directory
    outDir    : Output directory
    testName  : Name of test
    flashSite : Flash-X site

    Return:
    -------
    testSiteInfo : Site xml file
    """
    testInfo = str(testDir)+'/{0}.xml'.format(testName)
    testSiteInfo = str(outDir)+'/TestInfo{0}.xml'.format(testName)

    with open('{0}'.format(testSiteInfo), 'w') as testFile: testFile.write('<{0}>\n'.format(flashSite))
    os.system('cat {0} >> {1}'.format(testInfo,testSiteInfo))
    with open('{0}'.format(testSiteInfo), 'a') as testFile: testFile.write('</{0}>\n'.format(flashSite))

    return testInfo,testSiteInfo

def __flashTest(testDir,testDict,mainDict):
    """
    Run flashTest.py from resources/FlashTest

    Arguments:
    ----------
    Arguments:
    testDir   : Test directory
    testDict  : Test dictionary
    mainDict  : Main dictionary from Config file

    """
    # Create output directory if it does on exist

    for testName,testList in testDict.items():
        
        outDir = mainDict['pathToOutdir']+'/'+str(testName)

        os.system('mkdir -pv {0}'.format(outDir))
 
        testInfo,testSiteInfo = __getTestInfo(testDir,outDir,testName,mainDict['flashSite'])

        # run flashTest.py and with desired configuration
        #
        exit_status = os.system('python3 {0}/flashTest.py \
                                 -z {1} \
                                 -o {2} \
                                 -c {3}/Config \
                                 -i {4} \
                                 -v \
                                 -L \
                                 -s {5} \
                                 {6}'.format(os.path.dirname(resources.FlashTest.__file__),
                                             mainDict['pathToFlash'],
                                             mainDict['pathToOutdir']+'/'+str(testName),
                                             testDir,
                                             testSiteInfo,
                                             mainDict['flashSite'],
                                             " ".join(testList)))

        os.system('rm {0}'.format(testSiteInfo))

    # Handle errors
    # TODO: this is not working need to fix
    if exit_status != 0:
        print('FlashXTest returned exit status {0}'.format(exit_status))
        print('---------------------------------------------------------')
        raise ValueError

    else:
        print('FlashXTest reports SUCCESS')
